using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Diagnostics;

using Equis.JanusToolkit;

namespace Equis.Fish
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        private const int LAG = 50;    // use 50 ms of local lag
      
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Rectangle viewportRect;

        XYFlipTimeLine[] playerPosition;

        const int leftPlayer = 0;
        const int rightPlayer = 1;
        const int left = 0;
        const int right = 1;
        const int numPlayers = 2;

        Texture2D backgroundTexture;
        Texture2D mySprite;
        Texture2D otherSprite;

        int me;
        int other;

        const float playerVelocity = 0.1f; // pixels/ms

        // position and direction of local player
        float XPosition;
        float YPosition;
        int Direction;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            this.graphics.PreferredBackBufferWidth = 720;
            this.graphics.PreferredBackBufferHeight = 500;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {

            int i = Janus.ClientNumber;


            Console.WriteLine("Initializing");
            playerPosition = new XYFlipTimeLine[2];
            playerPosition[leftPlayer] = new XYFlipTimeLine("position-1");
            playerPosition[rightPlayer] = new XYFlipTimeLine("position-2");
              Console.WriteLine("TimeLines Created");

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            viewportRect = new Rectangle(0, 0,
            graphics.GraphicsDevice.Viewport.Width,
            graphics.GraphicsDevice.Viewport.Height);

            backgroundTexture = Content.Load<Texture2D>("Sprites\\UnderwaterBackground");



            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Console.WriteLine("Waiting for client number");
            int playernumber = Janus.ClientNumber;
            Console.WriteLine("Got client number");

            if (playernumber == 0)
            {
                me = leftPlayer;
                other = rightPlayer;
                XPosition = 100.0f;
                YPosition = 100.0f;
                Direction = right;
                mySprite = Content.Load<Texture2D>("Sprites\\greenfish");
                otherSprite = Content.Load<Texture2D>("Sprites\\orangefish");
            }
            else
            {
                me = rightPlayer;
                other = leftPlayer;
                XPosition = 400.0f;
                YPosition = 300.0f;
                Direction = left;
                mySprite = Content.Load<Texture2D>("Sprites\\orangefish");
                otherSprite = Content.Load<Texture2D>("Sprites\\greenfish");
            }

            ContentManager contentManager = new ContentManager(this.Services, @"Content\");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // calling Janus.Disconnect() here is optional
            // it causes the client to send a disconnect message to the timeline server
            // if it is not called, the client connection will automatically timeout after 5 seconds
            Janus.Disconnect();
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.  Do not use this with Janus </param>
        protected override void Update(GameTime gameTime)
        {
            Janus.StartUpdate();

            // Allows the game to exit
            KeyboardState keyboardState = Keyboard.GetState();
            if (keyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            float elapsedMS = (float)Janus.ElapsedMS();
            
            // don't do anything if elapsed ms = 0
            if (elapsedMS > 0)
            {

                // check which keys were pressed and update local position accordingly
                if (keyboardState.IsKeyDown(Keys.W) || keyboardState.IsKeyDown(Keys.Up))
                {
                    YPosition -= playerVelocity * elapsedMS;
                }

                if (keyboardState.IsKeyDown(Keys.S) || keyboardState.IsKeyDown(Keys.Down))
                {
                    YPosition += playerVelocity * elapsedMS;
                }

                if (keyboardState.IsKeyDown(Keys.A) || keyboardState.IsKeyDown(Keys.Left))
                {
                    XPosition -= playerVelocity * elapsedMS;
                    Direction = left;
                }

                if (keyboardState.IsKeyDown(Keys.D) || keyboardState.IsKeyDown(Keys.Right))
                {
                    XPosition += playerVelocity * elapsedMS;
                    Direction = right;
                }

                // store the new player position in the timeline and broadcast the position
                // to the other player
                // the position is stored LAG ms in the future - this implements Local Lag
                playerPosition[me].Set(LAG, new XYFlip(XPosition, YPosition, Direction));
            }

            base.Update(gameTime);

            Janus.EndUpdate();
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            Janus.Freeze();

            spriteBatch.Begin();

            spriteBatch.Draw(backgroundTexture, viewportRect, Color.White);

            // get the position of the remote player and draw it
            if (playerPosition[other].Ready())
            {
                XYFlip otherShip = playerPosition[other].Get(0);
                SpriteEffects otherFlip = new SpriteEffects();
                if (otherShip.Flip == left)
                {
                    otherFlip = SpriteEffects.None;
                }
                else
                {
                    otherFlip = SpriteEffects.FlipHorizontally;
                }

                spriteBatch.Draw(
                            otherSprite,
                            new Vector2(otherShip.X, otherShip.Y),
                            null,          // clip rect
                            Color.White,   // tint
                            0.0f,
                            Vector2.Zero,  // origin of rotation
                            1.0f,          // scale
                            otherFlip,     // horiz/vertical flip
                            0);            // depth value (0..1; 0 = front)
            }

            // get the position of the local player and draw it
            if (playerPosition[me].Ready())
            {
                XYFlip myShip = playerPosition[me].Get(0);
                SpriteEffects myFlip = new SpriteEffects();
                if (myShip.Flip == left)
                {
                    myFlip = SpriteEffects.None;
                }
                else
                {
                    myFlip = SpriteEffects.FlipHorizontally;
                }
                spriteBatch.Draw(
                        mySprite,
                        new Vector2(myShip.X, myShip.Y),
                        null,           // clip rect
                        Color.White,    // tint
                        0.0f,           // rotation               
                        Vector2.Zero,   // origin of rotation
                        1.0f,           // scale
                        myFlip,         // horiz/vertical flip
                        0);             // depth value (0..1; 0 = front)

            }

            spriteBatch.End();
            base.Draw(gameTime);
            Janus.UnFreeze();

        }
    }
}
