using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Diagnostics;

using Equis.JanusToolkit;
using Equis.JanusToolkit.StandardTimeLines;

namespace Equis.Tanks
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        private const int LAG = 0;
        private const int REMOTE_LAG = 1000;
      
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Rectangle viewportRect;

        TankStatusTimeLine[] playerPosition;
        IntTimeLine[] playerHealth;
        TankCommandTimeLine playerCommand;
        StringCommandTimeLine connectionCommand;

        const int leftPlayer = 0;
        const int rightPlayer = 1;
        const int numPlayers = rightPlayer - leftPlayer + 1;
        GameObject[] players;

        Texture2D backgroundTexture;
        Texture2D healthBar;
        Texture2D shootingLine;
        bool shooting = false;
        bool fire = false;

        int me;
        int other;

        int InitialHealthPoints = 10;
        const float playerVelocity = 0.1f; // pixels/ms
        const float playerRotationSpeed = MathHelper.TwoPi / 3000; // radians/ms

        Keys forwardKey;
        Keys backKey;
        Keys leftKey;
        Keys rightKey;
        Keys rotateLeftKey;
        Keys rotateRightKey;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            this.graphics.PreferredBackBufferWidth = 720;
            this.graphics.PreferredBackBufferHeight = 720;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            forwardKey = Keys.W;
            leftKey = Keys.A;
            rightKey = Keys.D;
            backKey = Keys.S;
            rotateLeftKey = Keys.Left;
            rotateRightKey = Keys.Right;

            playerPosition = new TankStatusTimeLine[2];
            playerPosition[leftPlayer] = new TankStatusTimeLine("position-1");
            playerPosition[rightPlayer] = new TankStatusTimeLine("position-2");
            playerPosition[leftPlayer].MaximumLength = 400;
            playerPosition[rightPlayer].MaximumLength = 400;

            playerHealth = new IntTimeLine[2];
            playerHealth[leftPlayer] = new IntTimeLine("health-1");
            playerHealth[rightPlayer] = new IntTimeLine("health-2");

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            Janus.Freeze();
            viewportRect = new Rectangle(0, 0,
            graphics.GraphicsDevice.Viewport.Width,
            graphics.GraphicsDevice.Viewport.Height);

            backgroundTexture = Content.Load<Texture2D>("Sprites\\background");
            healthBar = Content.Load<Texture2D>("Sprites\\HealthBar");
            shootingLine = Content.Load<Texture2D>("Sprites\\Shoot");

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            int playernumber = Janus.ClientNumber - 1;

            connectionCommand = new StringCommandTimeLine("client-" + (playernumber + 1));
            connectionCommand.Send(0, Convert.ToString(playernumber + 1));
            playerCommand = new TankCommandTimeLine("command-" + +(playernumber + 1));

            if (playernumber == 0)
            {
                me = leftPlayer;
                other = rightPlayer;
            }
            else
            {
                me = rightPlayer;
                other = leftPlayer;
            }

            players = new GameObject[numPlayers];
            for (int i = 0; i < numPlayers; i++)
            {
                if (i == 0)
                {
                    players[i] = new GameObject(Content.Load<Texture2D>("Sprites\\GreenTank"));
                }
                else
                {
                    players[i] = new GameObject(Content.Load<Texture2D>("Sprites\\SandTank"));
                }

                players[i].Velocity = playerVelocity;
            }


            ContentManager contentManager = new ContentManager(this.Services, @"Content\");
            Janus.UnFreeze();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            Janus.StartUpdate();

            // Allows the game to exit
            KeyboardState keyboardState = Keyboard.GetState();
            if (keyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            UpdatePlayerTank(gameTime, me);

            base.Update(gameTime);

            Janus.EndUpdate();
        }

        private void UpdatePlayerTank(GameTime gameTime, int playerId)
        {
            int elapsedMS = Janus.ElapsedMS();

            KeyboardState keyboardState = Keyboard.GetState();
            bool goForward = keyboardState.IsKeyDown(forwardKey);
            bool goLeft = keyboardState.IsKeyDown(leftKey);
            bool goRight = keyboardState.IsKeyDown(rightKey);
            bool goBack = keyboardState.IsKeyDown(backKey);
            bool rotateLeft = keyboardState.IsKeyDown(rotateLeftKey);
            bool rotateRight = keyboardState.IsKeyDown(rotateRightKey);

            float velocityX = 0.0f;
            float velocityY = 0.0f;
            float velocityRot = 0.0f;

            if (goForward)
            {
                velocityY = players[playerId].Velocity;
            }

            if (goBack)
            {
                velocityY = -players[playerId].Velocity;
            }

            if (goLeft)
            {
                velocityX = -players[playerId].Velocity;
            }

            if (goRight)
            {
                velocityX = players[playerId].Velocity;
            }

            if (rotateLeft)
            {
                velocityRot = -playerRotationSpeed;
            }

            if (rotateRight)
            {
                velocityRot = playerRotationSpeed;
            }

            // the shooting stuff
            fire = false;
            if (keyboardState.IsKeyDown(Keys.Space))
            {
                if (shooting == false)
                {
                    // can't shoot if you are dead
                    int health;
                    if (playerHealth[me].Ready())
                    {
                        health = playerHealth[me].Get(0);
                    }
                    else
                    {
                        health = InitialHealthPoints;
                    }
                    if (health > 0)
                    {
                        shooting = true;
                        fire = true;
                    }
                }
            }
            else
            {
                shooting = false;
            }
            
            // don't do a set if elapsed ms = 0
            if (elapsedMS > 0)
            {
                playerCommand.Set(LAG, new TankCommand(REMOTE_LAG, elapsedMS, 
                    velocityX, velocityY, velocityRot, shooting, fire, 0));
            }
        }



        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            Janus.Freeze();

            spriteBatch.Begin();

            spriteBatch.Draw(backgroundTexture, viewportRect, Color.White);
            Vector2 healthScaling = new Vector2(1.0f, 1.0f);
            TankStatus otherShip;
            if (playerPosition[other].Ready())
            {
                otherShip = playerPosition[other].Get(-REMOTE_LAG);

                players[other].Position = new Vector2(otherShip.X, otherShip.Y);
                players[other].Rotation = otherShip.Rotation;

                if (playerHealth[other].Ready())
                {
                    healthScaling.X = (float)playerHealth[other].Get(0) / 10f;
                }
                else
                {
                    healthScaling.X = 1.0f;
                }

                if (healthScaling.X > 0f)
                {

                    if (otherShip.Shooting)
                    {
                        Vector2 shootScaling = new Vector2(1f, 20f);
                        spriteBatch.Draw(
                            shootingLine,
                            new Vector2(otherShip.X, otherShip.Y),
                            null, // clip rect
                            Color.White,  // tint
                            players[other].Rotation,
                            new Vector2(0f, 100f),     // origin of rotation
                            shootScaling,   // scale
                            SpriteEffects.None, // horiz/vertical flip
                            0); // depth value (0..1; 0 = front)
                    }
                    
                    spriteBatch.Draw(
                        players[other].Sprite,
                        players[other].Position,
                        null, // clip rect
                        Color.White,  // tint
                        players[other].Rotation,
                        players[other].Centre,     // origin of rotation
                        1.0f,   // scale
                        SpriteEffects.None, // horiz/vertical flip
                        0); // depth value (0..1; 0 = front)

                    spriteBatch.Draw(
                        healthBar,
                        new Vector2(players[other].Position.X - 13f, players[other].Position.Y - 27f),
                        null, // clip rect
                        Color.White,  // tint
                        0.0f,
                        new Vector2(0f, 0f),     // origin of rotation
                        healthScaling,   // scale
                        SpriteEffects.None, // horiz/vertical flip
                        0); // depth value (0..1; 0 = front)

                } 

  
            }

            // Draw the local player
            if (playerPosition[me].Ready())
            {
                TankStatus myShip = playerPosition[me].GetBefore(-REMOTE_LAG);
                // get all the commands that have happened between remote lag and 0
                // and calculate the current position to be displayed
                List<Command<TankCommand>> commandList = playerCommand.GetRange(-REMOTE_LAG, 0);
                float positionX = myShip.X;
                float positionY = myShip.Y;
                float rotation = myShip.Rotation;
                bool shooting = myShip.Shooting;
                foreach (Command<TankCommand> command in commandList)
                {
                    TankCommand currentCommand = command.Val;
                    int duration = currentCommand.Duration;
                    positionY = positionY
                        + (float)(duration * currentCommand.VelocityX * Math.Sin(rotation))
                        - (float)(duration * currentCommand.VelocityY * Math.Cos(rotation));
                    positionX = positionX
                        + (float)(duration * currentCommand.VelocityX * Math.Cos(rotation))
                        + (float)(duration * currentCommand.VelocityY * Math.Sin(rotation));
                    rotation = rotation + duration * currentCommand.VelocityRot;
                    if (rotation > Math.PI * 2)
                    {
                        rotation -= (float)(Math.PI * 2);
                    }
                    if (rotation < 0.0)
                    {
                        rotation += (float)(Math.PI * 2);
                    }
                    shooting = currentCommand.Shooting;


                }

                if (playerHealth[me].Ready())
                {
                    healthScaling.X = (float)playerHealth[me].Get(0) / 10f;
                }
                else
                {
                    healthScaling.X = 1.0f;
                }

                // only draw if the player is alive
                if (healthScaling.X > 0.0f)
                {
                    players[me].Position = new Vector2(positionX, positionY);

                    if (shooting)
                    {
                        float distance = (players[me].Position - players[other].Position).Length();
                        // determine if it is a hit or not
                        Vector2 endPoint = new Vector2((float)Math.Sin(myShip.Rotation), -(float)Math.Cos(myShip.Rotation));
                        endPoint *= distance;
                        endPoint += players[me].Position;
                        float howClose = (endPoint - players[other].Position).Length();
                        if (howClose > 20f)
                        {
                            distance = 1000f;
                        }
                        Vector2 shootScaling = new Vector2(1f, (distance - 12f) / 100f);
                        spriteBatch.Draw(
                            shootingLine,
                            new Vector2(players[me].Position.X, players[me].Position.Y),
                            null, // clip rect
                            Color.White,  // tint
                            rotation,
                            new Vector2(0f, 100f),     // origin of rotation
                            shootScaling,   // scale
                            SpriteEffects.None, // horiz/vertical flip
                            0); // depth value (0..1; 0 = front)
                    }

                    spriteBatch.Draw(
                        players[me].Sprite,
                        players[me].Position,
                        null, // clip rect
                        Color.White,  // tint
                        rotation,
                        players[me].Centre,     // origin of rotation
                        1.0f,   // scale
                        SpriteEffects.None, // horiz/vertical flip
                        0); // depth value (0..1; 0 = front)

                    spriteBatch.Draw(
                        healthBar,
                        new Vector2(players[me].Position.X - 13f, players[me].Position.Y - 27f),
                        null, // clip rect
                        Color.White,  // tint
                        0.0f,
                        new Vector2(0f, 0f),     // origin of rotation
                        healthScaling,   // scale
                        SpriteEffects.None, // horiz/vertical flip
                        0); // depth value (0..1; 0 = front)

                }
            }

            spriteBatch.End();
            base.Draw(gameTime);
            Janus.UnFreeze();

        }
    }
}
