﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using Equis.JanusToolkit;
using Equis.JanusToolkit.StandardTimeLines;

namespace Equis.Tanks
{
    class Program
    {
        static int MaxClients = 3;
        static int InitialHealthPoints = 10;
        static float HitRadius = 20.0f;

        static Dictionary<int, StringCommandTimeLine> Clients = new Dictionary<int, StringCommandTimeLine>();
        static Dictionary<int, TankStatusTimeLine> Positions = new Dictionary<int, TankStatusTimeLine>();
        static Dictionary<int, IntTimeLine> Health = new Dictionary<int, IntTimeLine>();
        static Dictionary<int, TankCommandTimeLine> Commands = new Dictionary<int, TankCommandTimeLine>();

        static Random random = new Random();
       
        
        static void NewClient(object sender, Command<string> msg)
        {
            Console.WriteLine("A new player has joined, client number = {0}", msg.Val);

            // msg is the client number
            // create a status stream and a command stream for that client
            string clientNumber = msg.Val;
            int playerNumber = Convert.ToInt32(msg.Val);
            if (!Positions.ContainsKey(playerNumber))
            {
                Positions.Add(playerNumber, new TankStatusTimeLine("position-" + clientNumber));
                Commands.Add(playerNumber, new TankCommandTimeLine("command-" + clientNumber));
                Health.Add(playerNumber, new IntTimeLine("health-" + clientNumber));
                Commands[playerNumber].Command += new TankCommandTimeLine.CommandHandler(ProcessCommand);
            }

            // set the initial position for this player's tank
            float startX = (float)(600.0 * (0.7 * random.NextDouble() + 0.15));
            float startY = (float)(600.0 * (0.7 * random.NextDouble() + 0.15));
            Positions[playerNumber].Set(0, new TankStatus(startX, startY, 0.0f, false, 0));

            // initialize player's health points
            Health[playerNumber].Set(0, InitialHealthPoints);

        }

        // a command has arrived from one of the clients
        static void ProcessCommand(object sender, Command<TankCommand> v)
        {
            Janus.Freeze();

            // determine which player sent this command
            TankCommandTimeLine currentCommandTimeLine = (TankCommandTimeLine)sender;
            String[] nameParts = currentCommandTimeLine.Name.Split('-');
            int playerNumber = Convert.ToInt32(nameParts[1]);

            // get player's most recent position and calculate the new position after the command
            TankStatus currentPosition = Positions[playerNumber].Get(0);

            // get the command
            TankCommand currentCommand = v.Val;

            // calculate the new position based on the current position and velocity
            float positionY = currentPosition.Y 
                + (float) (currentCommand.Duration * currentCommand.VelocityX * Math.Sin(currentPosition.Rotation))
                - (float) (currentCommand.Duration * currentCommand.VelocityY * Math.Cos(currentPosition.Rotation));
            float positionX = currentPosition.X
                + (float)(currentCommand.Duration * currentCommand.VelocityX * Math.Cos(currentPosition.Rotation))
                + (float)(currentCommand.Duration * currentCommand.VelocityY * Math.Sin(currentPosition.Rotation));
            float rotation = currentPosition.Rotation + currentCommand.Duration * currentCommand.VelocityRot;
            if (rotation > Math.PI * 2)
            {
                rotation -= (float) (Math.PI * 2);
            }
            if (rotation < 0.0)
            {
                rotation += (float)(Math.PI * 2);
            }
            bool shooting = currentCommand.Shooting;
            Positions[playerNumber].Set(currentCommand.Time, new TankStatus(positionX, positionY, rotation, shooting, 0));

            // check if firing and if other tank is hit, reduce its health points
            if (currentCommand.Fire)
            {
                // get the position of the other tank, accounting for the remote lag
                foreach (KeyValuePair<int, TankStatusTimeLine> pair in Positions)
                {
                    if (pair.Key != playerNumber) // do not shoot yourself
                    {
                        if (pair.Value.Ready())
                        {
                            TankStatus otherPosition = pair.Value.Get(currentCommand.Time - currentCommand.RemoteLag);
                            float distance = (float)Math.Sqrt((currentPosition.X - otherPosition.X) * (currentPosition.X - otherPosition.X) + (currentPosition.Y - otherPosition.Y) * (currentPosition.Y - otherPosition.Y));
                            float endPointX = currentPosition.X + ((float)Math.Sin(currentPosition.Rotation) * distance);
                            float endPointY = currentPosition.Y - ((float)Math.Cos(currentPosition.Rotation) * distance);
                            float howClose = (float)Math.Sqrt((endPointX - otherPosition.X) * (endPointX - otherPosition.X) + (endPointY - otherPosition.Y) * (endPointY - otherPosition.Y));

                            int currentHealth;
                            if (howClose <= HitRadius)
                            {
                                if (Health[pair.Key].Ready())
                                {
                                    currentHealth = Health[pair.Key].Get(0);
                                }
                                else
                                {
                                    currentHealth = InitialHealthPoints;
                                }
                                Health[pair.Key].Set(0, currentHealth - 1);
                            }
                        }
                    }
             
                
                }
            }
            Janus.UnFreeze();
        }

        static void Main(string[] args)
        {

            TimeLineServer tls = new TimeLineServer();
            tls.Start();

            
            // create a command stream for each potential client
            // when a client joins, all the other streams for that client will be created

            for (int i = 0; i < MaxClients; i++)
            {
                Clients.Add(i, new StringCommandTimeLine(  "client-" + Convert.ToString(i)));
                Clients[i].Command += new StringCommandTimeLine.CommandHandler(NewClient);
            }
            Console.WriteLine("Server Started - waiting for clients");

            // do nothing, but keep the process alive...
            while (true)
            {
                Thread.Sleep(5000);
            }

        }
    }
}
