﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Equis.JanusToolkit;
using Equis.JanusToolkit.StandardTimeLines;

// Simple telepointer example

namespace Equis.Telepointer
{
    public partial class Form1 : Form
    {
        private const int LAG = 0;                  // local lag amount
        private const int MAX_CLIENTS = 5;          // maximum number of clients

        PositionTimeLine[] pointerPosition;         // array of timeline objects
        int myClientNumber;                         // client number for local client

        public Form1()
        {
            InitializeComponent();
        }

        // store a new position in the timeline everytime the mouse moves
        private void Form1_MouseMove(object sender, MouseEventArgs mouseEv)
        {
            pointerPosition[myClientNumber].Set(LAG, new Position(mouseEv.X, mouseEv.Y));
        }

        // create a timeline for each of the possible clients
        private void Form1_Load(object sender, EventArgs e)
        {
            pointerPosition = new PositionTimeLine[MAX_CLIENTS];
            for (int i = 0; i < MAX_CLIENTS; i++)
            {
                pointerPosition[i] = new PositionTimeLine("pointer-" + i);
                pointerPosition[i].ExtrapolationType = PositionTimeLine.stepping;
            }
            // client numbers are unique and start at 0
            myClientNumber = Janus.ClientNumber;
        }

        // Get the positions of all telepointers and draw them
        private void timer1_Tick(object sender, EventArgs e)
        {
            Graphics gfx = CreateGraphics();
            gfx.Clear(Color.SeaShell);

            Position cursorPosition;
            for (int i = 0; i < MAX_CLIENTS; i++)
            {
                if (pointerPosition[i] != null)
                {
                   if (pointerPosition[i].Ready()) // only draw if there are values in the timeline
                   {
                       cursorPosition = pointerPosition[i].Get(0);
                       // make me Red, all others Blue
                       Color tpc = i == myClientNumber ? Color.Red : Color.Blue;
                       gfx.DrawRectangle(new Pen(tpc), cursorPosition.X, cursorPosition.Y, 5, 5);
                   }
                }
            }
        }
    }
}
