﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Diagnostics;

using System.Windows.Forms;
using System.Drawing;



namespace Equis.JanusToolkit
{

    /// <summary>
    /// Input box with prompt and input validation
    /// </summary>
    public class InputBox
    {
        /// <summary>
        /// Display input box
        /// </summary>
        /// <param name="title">Input box title</param>
        /// <param name="promptText">Prompt text</param>
        /// <param name="value">Initial value</param>
        public static DialogResult Show(string title, string promptText, ref string value)
        {
            return Show(title, promptText, ref value, null);
        }

        /// <summary>
        /// Display input box
        /// </summary>
        /// <param name="title">Input box title</param>
        /// <param name="promptText">Prompt text</param>
        /// <param name="value">Initial value</param>
        /// <param name="validation">Input box validation delegate</param>
        public static DialogResult Show(string title, string promptText, ref string value,
                                        InputBoxValidation validation)
        {
            Application.EnableVisualStyles();

            Form form = new Form();
            Label label = new Label();
            TextBox textBox = new TextBox();
            Button buttonYes = new Button();
            Button buttonNo = new Button();

            form.Text = title;
            label.Text = promptText;
            textBox.Text = value;

            buttonYes.Text = "Yes";
            buttonNo.Text = "No";
            buttonYes.DialogResult = DialogResult.Yes;
            buttonNo.DialogResult = DialogResult.No;

            label.SetBounds(9, 20, 372, 13);
            textBox.SetBounds(12, 44, 372, 20);
            buttonYes.SetBounds(228, 77, 75, 23);
            buttonNo.SetBounds(309, 77, 75, 23);

            label.AutoSize = true;
            textBox.Anchor = textBox.Anchor | AnchorStyles.Right;
            buttonYes.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;
            buttonNo.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;

            form.ClientSize = new Size(396, 112);
            form.Controls.AddRange(new Control[] { label, textBox, buttonYes, buttonNo });
            form.ClientSize = new Size(Math.Max(300, label.Right + 10), form.ClientSize.Height);
            form.FormBorderStyle = FormBorderStyle.FixedDialog;
            form.StartPosition = FormStartPosition.CenterScreen;
            form.MinimizeBox = false;
            form.MaximizeBox = false;
            form.AcceptButton = buttonYes;
            form.CancelButton = buttonNo;

            if (validation != null)
            {
                form.FormClosing += delegate(object sender, FormClosingEventArgs e)
                {
                    if (form.DialogResult == DialogResult.Yes)
                    {
                        string errorText = validation(textBox.Text);
                        if (e.Cancel = (errorText != ""))
                        {
                            MessageBox.Show(form, errorText, "Validation Error",
                                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                            textBox.Focus();
                        }
                    }
                };
            }

            DialogResult dialogResult = form.ShowDialog();
            value = textBox.Text;
            return dialogResult;
        }
    }

    /// <summary>
    /// Delegate for input validation
    /// </summary>
    public delegate string InputBoxValidation(string errorMessage);




}