﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;

using Lidgren.Network;


namespace Equis.JanusToolkit
{
    
    ///
    /// For each connection records the client number assigned to that socket
    ///
    class ConnectionInfo
    {

        struct ClockSync
        {
            public long rtt; // round trip time
            public long clockOffset;  // offset from server clock

            public ClockSync(long _rtt, long _offset)
            {
                rtt = _rtt;
                clockOffset = _offset;
            }
        }


        private Dictionary<NetConnection, int> _clients;
        private Dictionary<NetConnection, List<ClockSync>> _clockSync;
        private List<NetConnection> _clientList;

        public void AddConnection(NetConnection clientConnection)
        {
            int clientNumber = 0;
            while (_clients.ContainsValue(clientNumber))
            {
                clientNumber++;
            }
            _clients.Add(clientConnection, clientNumber);
            _clientList.Add(clientConnection);
        }

        public void RemoveConnection(NetConnection clientConnection)
        {
            if (_clients.ContainsKey(clientConnection))
            {
                _clients.Remove(clientConnection);
                _clientList.Remove(clientConnection);

            }

        }

        public void AddClockSync(NetConnection clientConnection, long rtt, long offset)
        {
            if (_clockSync.ContainsKey(clientConnection))
            {
                List<ClockSync> syncList = _clockSync[clientConnection];
                syncList.Add(new ClockSync(rtt, offset));
                if (syncList.Count > 12)
                {
                    syncList.RemoveAt(0);
                }
                _clockSync[clientConnection] = syncList;

            }
            else
            {               
                List<ClockSync> thisSyncAsList = new List<ClockSync>();
                thisSyncAsList.Add(new ClockSync(rtt, offset));
                _clockSync.Add(clientConnection, thisSyncAsList);
            }
        }

        public long ClockOffset(NetConnection clientConnection)
        {
            int numBest = 6;
            long maxBestRtt = 0;
            List<ClockSync> bestSyncList = new List<ClockSync>();
            int i = 0;
            foreach (ClockSync cs in _clockSync[clientConnection])
            {
                if (i < numBest)
                {
                    bestSyncList.Add(cs);
                    if (cs.rtt > maxBestRtt)
                    {
                        maxBestRtt = cs.rtt;
                    }
                    i++;
                }
                else
                {
                    if (cs.rtt < maxBestRtt)
                    {
                        int j = 0;
                        int indexToRemove = -999;
                        foreach (ClockSync csb in bestSyncList)
                        {
                            if (csb.rtt == maxBestRtt)
                            {
                                indexToRemove = j;
                            }
                            j++;
                        }
                        bestSyncList.RemoveAt(indexToRemove);
                        bestSyncList.Add(cs);
                        // get new max best
                        maxBestRtt = 0;
                        foreach (ClockSync csb in bestSyncList)
                        {
                            if (csb.rtt > maxBestRtt)
                            {
                                maxBestRtt = csb.rtt;
                            }
                            j++;
                        }


                    }

                }
            }
            // calc the average of the 10 best
            long average = 0;
            foreach (ClockSync csb in bestSyncList)
            {
                average += csb.clockOffset;
            }
            average = average / bestSyncList.Count();
            return average;

        }


        public int GetClientNumber(NetConnection clientConnection)
        {
            return _clients[clientConnection];
        }

        public List<NetConnection> ClientIdentifiers()
        {
            return _clientList;
        }

        public ConnectionInfo()
        {
            _clients = new Dictionary<NetConnection, int>();
            _clockSync = new Dictionary<NetConnection, List<ClockSync>>();
            _clientList = new List<NetConnection>();
        }
    }
}
